/* Help string externs.

   Copyright (C) 1993 Sebastiano Vigna

    This file is part of ne, the nice editor.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

In other words, you are welcome to use, share and improve this program.
You are forbidden to forbid anyone else to use, share and improve
what you give them.   Help stamp out software-hoarding!  */


extern const unsigned char * const CLEAR_HELP[7];
extern const unsigned char * const OPEN_HELP[18];
extern const unsigned char * const SAVE_HELP[12];
extern const unsigned char * const SAVEAS_HELP[14];
extern const unsigned char * const QUIT_HELP[6];
extern const unsigned char * const EXIT_HELP[7];
extern const unsigned char * const NEWDOC_HELP[8];
extern const unsigned char * const CLOSEDOC_HELP[8];
extern const unsigned char * const NEXTDOC_HELP[5];
extern const unsigned char * const PREVDOC_HELP[5];
extern const unsigned char * const SELECTDOC_HELP[14];
extern const unsigned char * const MARK_HELP[14];
extern const unsigned char * const MARKVERT_HELP[10];
extern const unsigned char * const COPY_HELP[10];
extern const unsigned char * const CUT_HELP[5];
extern const unsigned char * const PASTE_HELP[7];
extern const unsigned char * const PASTEVERT_HELP[7];
extern const unsigned char * const ERASE_HELP[6];
extern const unsigned char * const OPENCLIP_HELP[14];
extern const unsigned char * const SAVECLIP_HELP[13];
extern const unsigned char * const CLIPNUMBER_HELP[11];
extern const unsigned char * const FIND_HELP[11];
extern const unsigned char * const FINDREGEXP_HELP[12];
extern const unsigned char * const REPLACE_HELP[46];
extern const unsigned char * const REPLACEONCE_HELP[10];
extern const unsigned char * const REPLACEALL_HELP[12];
extern const unsigned char * const REPEATLAST_HELP[26];
extern const unsigned char * const MATCHBRACKET_HELP[8];
extern const unsigned char * const SEARCHBACK_HELP[14];
extern const unsigned char * const CASESEARCH_HELP[13];
extern const unsigned char * const RECORD_HELP[20];
extern const unsigned char * const PLAY_HELP[16];
extern const unsigned char * const MACRO_HELP[29];
extern const unsigned char * const OPENMACRO_HELP[14];
extern const unsigned char * const SAVEMACRO_HELP[17];
extern const unsigned char * const UNLOADMACROS_HELP[12];
extern const unsigned char * const UNDO_HELP[7];
extern const unsigned char * const REDO_HELP[6];
extern const unsigned char * const UNDELLINE_HELP[15];
extern const unsigned char * const DOUNDO_HELP[19];
extern const unsigned char * const CENTER_HELP[8];
extern const unsigned char * const PARAGRAPH_HELP[15];
extern const unsigned char * const TOUPPER_HELP[15];
extern const unsigned char * const TOLOWER_HELP[5];
extern const unsigned char * const CAPITALIZE_HELP[6];
extern const unsigned char * const RIGHTMARGIN_HELP[12];
extern const unsigned char * const WORDWRAP_HELP[12];
extern const unsigned char * const AUTOINDENT_HELP[21];
extern const unsigned char * const AUTOPREFS_HELP[15];
extern const unsigned char * const BINARY_HELP[22];
extern const unsigned char * const INSERT_HELP[11];
extern const unsigned char * const FASTGUI_HELP[14];
extern const unsigned char * const FREEFORM_HELP[17];
extern const unsigned char * const NOFILEREQ_HELP[10];
extern const unsigned char * const STATUSBAR_HELP[18];
extern const unsigned char * const READONLY_HELP[13];
extern const unsigned char * const TABSIZE_HELP[9];
extern const unsigned char * const TURBO_HELP[29];
extern const unsigned char * const VERBOSEMACROS_HELP[18];
extern const unsigned char * const LOADPREFS_HELP[19];
extern const unsigned char * const SAVEPREFS_HELP[14];
extern const unsigned char * const LOADAUTOPREFS_HELP[7];
extern const unsigned char * const SAVEAUTOPREFS_HELP[7];
extern const unsigned char * const SAVEDEFPREFS_HELP[6];
extern const unsigned char * const MOVELEFT_HELP[6];
extern const unsigned char * const MOVERIGHT_HELP[6];
extern const unsigned char * const LINEUP_HELP[6];
extern const unsigned char * const LINEDOWN_HELP[6];
extern const unsigned char * const GOTOLINE_HELP[10];
extern const unsigned char * const GOTOCOLUMN_HELP[8];
extern const unsigned char * const GOTOMARK_HELP[9];
extern const unsigned char * const PREVPAGE_HELP[8];
extern const unsigned char * const NEXTPAGE_HELP[8];
extern const unsigned char * const PREVWORD_HELP[8];
extern const unsigned char * const NEXTWORD_HELP[6];
extern const unsigned char * const MOVEEOL_HELP[5];
extern const unsigned char * const MOVESOL_HELP[5];
extern const unsigned char * const MOVEEOF_HELP[5];
extern const unsigned char * const MOVESOF_HELP[5];
extern const unsigned char * const MOVEEOW_HELP[8];
extern const unsigned char * const TOGGLESEOF_HELP[10];
extern const unsigned char * const TOGGLESEOL_HELP[10];
extern const unsigned char * const SETBOOKMARK_HELP[7];
extern const unsigned char * const GOTOBOOKMARK_HELP[7];
extern const unsigned char * const INSERTCHAR_HELP[13];
extern const unsigned char * const DELETECHAR_HELP[9];
extern const unsigned char * const BACKSPACE_HELP[6];
extern const unsigned char * const INSERTLINE_HELP[7];
extern const unsigned char * const DELETELINE_HELP[8];
extern const unsigned char * const DELETEEOL_HELP[10];
extern const unsigned char * const ABOUT_HELP[5];
extern const unsigned char * const BEEP_HELP[6];
extern const unsigned char * const EXEC_HELP[13];
extern const unsigned char * const FLASH_HELP[6];
extern const unsigned char * const HELP_HELP[13];
extern const unsigned char * const NOP_HELP[5];
extern const unsigned char * const REFRESH_HELP[8];
extern const unsigned char * const SYSTEM_HELP[7];
extern const unsigned char * const ESCAPE_HELP[7];
