/* Names and abbreviations of all the commands.

   Copyright (C) 1993 Sebastiano Vigna

    This file is part of ne, the nice editor.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

In other words, you are welcome to use, share and improve this program.
You are forbidden to forbid anyone else to use, share and improve
what you give them.   Help stamp out software-hoarding!  */

#include "ne.h"

/* Here we have the name of all commands. They are extern'd in names.h. Note
that whenever you add or remove a command, you should immediately change also
the command_names array at the end of this file. */


const unsigned char NOP_NAME[] = "NOP";
const unsigned char NOP_ABBREV[] = "NOP";


const unsigned char OPEN_NAME[] = "Open";
const unsigned char OPEN_ABBREV[] = "O";

const unsigned char SAVE_NAME[] = "Save";
const unsigned char SAVE_ABBREV[] = "S";

const unsigned char SAVEAS_NAME[] = "SaveAs";
const unsigned char SAVEAS_ABBREV[] = "SA";

const unsigned char CLEAR_NAME[] = "Clear";
const unsigned char CLEAR_ABBREV[] = "CL";

const unsigned char QUIT_NAME[] = "Quit";
const unsigned char QUIT_ABBREV[] = "Q";

const unsigned char EXIT_NAME[] = "Exit";
const unsigned char EXIT_ABBREV[] = "X";

const unsigned char ABOUT_NAME[] = "About";
const unsigned char ABOUT_ABBREV[] = "About";


const unsigned char NEWDOC_NAME[] = "NewDoc";
const unsigned char NEWDOC_ABBREV[] = "N";

const unsigned char CLOSEDOC_NAME[] = "CloseDoc";
const unsigned char CLOSEDOC_ABBREV[] = "CD";

const unsigned char NEXTDOC_NAME[] = "NextDoc";
const unsigned char NEXTDOC_ABBREV[] = "ND";

const unsigned char PREVDOC_NAME[] = "PrevDoc";
const unsigned char PREVDOC_ABBREV[] = "PD";

const unsigned char SELECTDOC_NAME[] = "SelectDoc";
const unsigned char SELECTDOC_ABBREV[] = "SD";


const unsigned char MARK_NAME[] = "Mark";
const unsigned char MARK_ABBREV[] = "M";

const unsigned char CUT_NAME[] = "Cut";
const unsigned char CUT_ABBREV[] = "CU";

const unsigned char COPY_NAME[] = "Copy";
const unsigned char COPY_ABBREV[] = "C";

const unsigned char PASTE_NAME[] = "Paste";
const unsigned char PASTE_ABBREV[] = "P";

const unsigned char ERASE_NAME[] = "Erase";
const unsigned char ERASE_ABBREV[] = "E";

const unsigned char MARKVERT_NAME[] = "MarkVert";
const unsigned char MARKVERT_ABBREV[] = "MV";

const unsigned char PASTEVERT_NAME[] = "PasteVert";
const unsigned char PASTEVERT_ABBREV[] = "PV";

const unsigned char OPENCLIP_NAME[] = "OpenClip";
const unsigned char OPENCLIP_ABBREV[] = "OC";

const unsigned char SAVECLIP_NAME[] = "SaveClip";
const unsigned char SAVECLIP_ABBREV[] = "SC";


const unsigned char FIND_NAME[] = "Find";
const unsigned char FIND_ABBREV[] = "F";

const unsigned char FINDREGEXP_NAME[] = "FindRegExp";
const unsigned char FINDREGEXP_ABBREV[] = "FX";

const unsigned char REPLACE_NAME[] = "Replace";
const unsigned char REPLACE_ABBREV[] = "R";

const unsigned char REPLACEONCE_NAME[] = "ReplaceOnce";
const unsigned char REPLACEONCE_ABBREV[] = "R1";

const unsigned char REPLACEALL_NAME[] = "ReplaceAll";
const unsigned char REPLACEALL_ABBREV[] = "RA";

const unsigned char REPEATLAST_NAME[] = "RepeatLast";
const unsigned char REPEATLAST_ABBREV[] = "RL";

const unsigned char GOTOLINE_NAME[] = "GotoLine";
const unsigned char GOTOLINE_ABBREV[] = "GL";

const unsigned char GOTOCOLUMN_NAME[] = "GotoColumn";
const unsigned char GOTOCOLUMN_ABBREV[] = "GC";

const unsigned char GOTOMARK_NAME[] = "GotoMark";
const unsigned char GOTOMARK_ABBREV[] = "GM";

const unsigned char MATCHBRACKET_NAME[] = "MatchBracket";
const unsigned char MATCHBRACKET_ABBREV[] = "MB";


const unsigned char RECORD_NAME[] = "Record";
const unsigned char RECORD_ABBREV[] = "Rec";

const unsigned char PLAY_NAME[] = "Play";
const unsigned char PLAY_ABBREV[] = "PL";

/* This is an extra which is very useful in the default menus and key bindings. */

const unsigned char PLAYONCE_ABBREV[] = "PL 1";

const unsigned char OPENMACRO_NAME[] = "OpenMacro";
const unsigned char OPENMACRO_ABBREV[] = "OM";

const unsigned char SAVEMACRO_NAME[] = "SaveMacro";
const unsigned char SAVEMACRO_ABBREV[] = "SM";

const unsigned char MACRO_NAME[] = "Macro";
const unsigned char MACRO_ABBREV[] = "MA";

const unsigned char UNLOADMACROS_NAME[] = "UnloadMacros";
const unsigned char UNLOADMACROS_ABBREV[] = "UM";


const unsigned char REFRESH_NAME[] = "Refresh";
const unsigned char REFRESH_ABBREV[] = "REF";

const unsigned char UNDELLINE_NAME[] = "UndelLine";
const unsigned char UNDELLINE_ABBREV[] = "UL";

const unsigned char CENTER_NAME[] = "Center";
const unsigned char CENTER_ABBREV[] = "CE";

const unsigned char PARAGRAPH_NAME[] = "Paragraph";
const unsigned char PARAGRAPH_ABBREV[] = "PA";

const unsigned char TOUPPER_NAME[] = "ToUpper";
const unsigned char TOUPPER_ABBREV[] = "TU";

const unsigned char TOLOWER_NAME[] = "ToLower";
const unsigned char TOLOWER_ABBREV[] = "TL";

const unsigned char CAPITALIZE_NAME[] = "Capitalize";
const unsigned char CAPITALIZE_ABBREV[] = "CA";


const unsigned char MOVELEFT_NAME[] = "MoveLeft";
const unsigned char MOVELEFT_ABBREV[] = "ML";

const unsigned char MOVERIGHT_NAME[] = "MoveRight";
const unsigned char MOVERIGHT_ABBREV[] = "MR";

const unsigned char LINEUP_NAME[] = "LineUp";
const unsigned char LINEUP_ABBREV[] = "LU";

const unsigned char LINEDOWN_NAME[] = "LineDown";
const unsigned char LINEDOWN_ABBREV[] = "LD";

const unsigned char PREVPAGE_NAME[] = "PrevPage";
const unsigned char PREVPAGE_ABBREV[] = "PP";

const unsigned char NEXTPAGE_NAME[] = "NextPage";
const unsigned char NEXTPAGE_ABBREV[] = "NP";

const unsigned char MOVEEOL_NAME[] = "MoveEOL";
const unsigned char MOVEEOL_ABBREV[] = "EOL";

const unsigned char MOVESOL_NAME[] = "MoveSOL";
const unsigned char MOVESOL_ABBREV[] = "SOL";

const unsigned char MOVEEOF_NAME[] = "MoveEOF";
const unsigned char MOVEEOF_ABBREV[] = "EOF";

const unsigned char MOVESOF_NAME[] = "MoveSOF";
const unsigned char MOVESOF_ABBREV[] = "SOF";

const unsigned char TOGGLESEOF_NAME[] = "ToggleSEOF";
const unsigned char TOGGLESEOF_ABBREV[] = "TSEOF";

const unsigned char TOGGLESEOL_NAME[] = "ToggleSEOL";
const unsigned char TOGGLESEOL_ABBREV[] = "TSEOL";

const unsigned char NEXTWORD_NAME[] = "NextWord";
const unsigned char NEXTWORD_ABBREV[] = "NW";

const unsigned char PREVWORD_NAME[] = "PrevWord";
const unsigned char PREVWORD_ABBREV[] = "PW";

const unsigned char MOVEEOW_NAME[] = "MoveEOW";
const unsigned char MOVEEOW_ABBREV[] = "EOW";

const unsigned char SETBOOKMARK_NAME[] = "SetBookmark";
const unsigned char SETBOOKMARK_ABBREV[] = "SBM";

const unsigned char GOTOBOOKMARK_NAME[] = "GotoBookmark";
const unsigned char GOTOBOOKMARK_ABBREV[] = "GBM";



const unsigned char TABSIZE_NAME[] = "TABSize";
const unsigned char TABSIZE_ABBREV[] = "TS";

const unsigned char INSERT_NAME[] = "Insert";
const unsigned char INSERT_ABBREV[] = "I";

const unsigned char FREEFORM_NAME[] = "FreeForm";
const unsigned char FREEFORM_ABBREV[] = "FF";

const unsigned char STATUSBAR_NAME[] = "StatusBar";
const unsigned char STATUSBAR_ABBREV[] = "ST";

const unsigned char SEARCHBACK_NAME[] = "SearchBack";
const unsigned char SEARCHBACK_ABBREV[] = "SB";

const unsigned char CASESEARCH_NAME[] = "CaseSearch";
const unsigned char CASESEARCH_ABBREV[] = "CS";

const unsigned char FASTGUI_NAME[] = "FastGUI";
const unsigned char FASTGUI_ABBREV[] = "FG";

const unsigned char WORDWRAP_NAME[] = "WordWrap";
const unsigned char WORDWRAP_ABBREV[] = "WW";

const unsigned char RIGHTMARGIN_NAME[] = "RightMargin";
const unsigned char RIGHTMARGIN_ABBREV[] = "RM";

const unsigned char AUTOINDENT_NAME[] = "AutoIndent";
const unsigned char AUTOINDENT_ABBREV[] = "AI";

const unsigned char AUTOPREFS_NAME[] = "AutoPrefs";
const unsigned char AUTOPREFS_ABBREV[] = "AP";

const unsigned char BINARY_NAME[] = "Binary";
const unsigned char BINARY_ABBREV[] = "B";

const unsigned char TURBO_NAME[] = "Turbo";
const unsigned char TURBO_ABBREV[] = "T";

const unsigned char NOFILEREQ_NAME[] = "NoFileReq";
const unsigned char NOFILEREQ_ABBREV[] = "NFR";

const unsigned char VERBOSEMACROS_NAME[] = "VerboseMacros";
const unsigned char VERBOSEMACROS_ABBREV[] = "VM";

const unsigned char DOUNDO_NAME[] = "DoUndo";
const unsigned char DOUNDO_ABBREV[] = "DU";

const unsigned char READONLY_NAME[] = "ReadOnly";
const unsigned char READONLY_ABBREV[] = "RO";

const unsigned char CLIPNUMBER_NAME[] = "ClipNumber";
const unsigned char CLIPNUMBER_ABBREV[] = "CN";

const unsigned char LOADPREFS_NAME[] = "LoadPrefs";
const unsigned char LOADPREFS_ABBREV[] = "LP";

const unsigned char SAVEPREFS_NAME[] = "SavePrefs";
const unsigned char SAVEPREFS_ABBREV[] = "SP";

const unsigned char LOADAUTOPREFS_NAME[] = "LoadAutoPrefs";
const unsigned char LOADAUTOPREFS_ABBREV[] = "LAP";

const unsigned char SAVEAUTOPREFS_NAME[] = "SaveAutoPrefs";
const unsigned char SAVEAUTOPREFS_ABBREV[] = "SAP";

const unsigned char SAVEDEFPREFS_NAME[] = "SaveDefPrefs";
const unsigned char SAVEDEFPREFS_ABBREV[] = "SDP";


const unsigned char INSERTCHAR_NAME[] = "InsertChar";
const unsigned char INSERTCHAR_ABBREV[] = "IC";

const unsigned char DELETECHAR_NAME[] = "DeleteChar";
const unsigned char DELETECHAR_ABBREV[] = "DC";

const unsigned char BACKSPACE_NAME[] = "BackSpace";
const unsigned char BACKSPACE_ABBREV[] = "BS";

const unsigned char INSERTLINE_NAME[] = "InsertLine";
const unsigned char INSERTLINE_ABBREV[] = "IL";

const unsigned char DELETELINE_NAME[] = "DeleteLine";
const unsigned char DELETELINE_ABBREV[] = "DL";

const unsigned char DELETEEOL_NAME[] = "DeleteEOL";
const unsigned char DELETEEOL_ABBREV[] = "DE";


const unsigned char UNDO_NAME[] = "Undo";
const unsigned char UNDO_ABBREV[] = "U";

const unsigned char REDO_NAME[] = "Redo";
const unsigned char REDO_ABBREV[] = "RE";


const unsigned char HELP_NAME[] = "Help";
const unsigned char HELP_ABBREV[] = "H";

const unsigned char BEEP_NAME[] = "Beep";
const unsigned char BEEP_ABBREV[] = "BE";

const unsigned char FLASH_NAME[] = "Flash";
const unsigned char FLASH_ABBREV[] = "FL";

const unsigned char EXEC_NAME[] = "Exec";
const unsigned char EXEC_ABBREV[] = "EX";

const unsigned char SYSTEM_NAME[] = "System";
const unsigned char SYSTEM_ABBREV[] = "SYS";

const unsigned char ESCAPE_NAME[] = "Escape";
const unsigned char ESCAPE_ABBREV[] = "ESC";


/* This is the NULL-terminated, ordered list of names, useful for help etc. */

const unsigned char * const command_names[ACTION_COUNT+1] = {

	ABOUT_NAME, AUTOINDENT_NAME, AUTOPREFS_NAME, BACKSPACE_NAME, BEEP_NAME,
	BINARY_NAME, CAPITALIZE_NAME, CASESEARCH_NAME, CENTER_NAME, CLEAR_NAME,
	CLIPNUMBER_NAME, CLOSEDOC_NAME, COPY_NAME, CUT_NAME, DELETECHAR_NAME,
	DELETEEOL_NAME, DELETELINE_NAME, DOUNDO_NAME, ERASE_NAME, ESCAPE_NAME,
	EXEC_NAME, EXIT_NAME, FASTGUI_NAME, FINDREGEXP_NAME, FIND_NAME, FLASH_NAME,
	FREEFORM_NAME, GOTOBOOKMARK_NAME, GOTOCOLUMN_NAME, GOTOLINE_NAME,
	GOTOMARK_NAME, HELP_NAME, INSERTCHAR_NAME, INSERTLINE_NAME, INSERT_NAME,
	LINEDOWN_NAME, LINEUP_NAME, LOADAUTOPREFS_NAME, LOADPREFS_NAME, MACRO_NAME,
	MARKVERT_NAME, MARK_NAME, MATCHBRACKET_NAME, MOVEEOF_NAME, MOVEEOL_NAME,
	MOVEEOW_NAME, MOVELEFT_NAME, MOVERIGHT_NAME, MOVESOF_NAME, MOVESOL_NAME,
	NEWDOC_NAME, NEXTDOC_NAME, NEXTPAGE_NAME, NEXTWORD_NAME, NOFILEREQ_NAME,
	NOP_NAME, OPENCLIP_NAME, OPENMACRO_NAME, OPEN_NAME, PARAGRAPH_NAME,
	PASTEVERT_NAME, PASTE_NAME, PLAY_NAME, PREVDOC_NAME, PREVPAGE_NAME,
	PREVWORD_NAME, QUIT_NAME, READONLY_NAME, RECORD_NAME, REDO_NAME, REFRESH_NAME,
	REPEATLAST_NAME, REPLACEALL_NAME, REPLACEONCE_NAME, REPLACE_NAME,
	RIGHTMARGIN_NAME, SAVEAS_NAME, SAVEAUTOPREFS_NAME, SAVECLIP_NAME,
	SAVEDEFPREFS_NAME, SAVEMACRO_NAME, SAVEPREFS_NAME, SAVE_NAME, SEARCHBACK_NAME,
	SELECTDOC_NAME, SETBOOKMARK_NAME, STATUSBAR_NAME, SYSTEM_NAME, TABSIZE_NAME,
	TOGGLESEOF_NAME, TOGGLESEOL_NAME, TOLOWER_NAME, TOUPPER_NAME, TURBO_NAME,
	UNDELLINE_NAME, UNDO_NAME, UNLOADMACROS_NAME, VERBOSEMACROS_NAME,
	WORDWRAP_NAME, NULL

};
