/* Extern's for names and abbreviations of all the commands.

   Copyright (C) 1993 Sebastiano Vigna

    This file is part of ne, the nice editor.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

In other words, you are welcome to use, share and improve this program.
You are forbidden to forbid anyone else to use, share and improve
what you give them.   Help stamp out software-hoarding!  */


extern const unsigned char NOP_NAME[];
extern const unsigned char NOP_ABBREV[];


extern const unsigned char OPEN_NAME[];
extern const unsigned char OPEN_ABBREV[];

extern const unsigned char SAVE_NAME[];
extern const unsigned char SAVE_ABBREV[];

extern const unsigned char SAVEAS_NAME[];
extern const unsigned char SAVEAS_ABBREV[];

extern const unsigned char CLEAR_NAME[];
extern const unsigned char CLEAR_ABBREV[];

extern const unsigned char QUIT_NAME[];
extern const unsigned char QUIT_ABBREV[];

extern const unsigned char EXIT_NAME[];
extern const unsigned char EXIT_ABBREV[];

extern const unsigned char ABOUT_NAME[];
extern const unsigned char ABOUT_ABBREV[];


extern const unsigned char NEWDOC_NAME[];
extern const unsigned char NEWDOC_ABBREV[];

extern const unsigned char CLOSEDOC_NAME[];
extern const unsigned char CLOSEDOC_ABBREV[];

extern const unsigned char NEXTDOC_NAME[];
extern const unsigned char NEXTDOC_ABBREV[];

extern const unsigned char PREVDOC_NAME[];
extern const unsigned char PREVDOC_ABBREV[];

extern const unsigned char SELECTDOC_NAME[];
extern const unsigned char SELECTDOC_ABBREV[];


extern const unsigned char MARK_NAME[];
extern const unsigned char MARK_ABBREV[];

extern const unsigned char CUT_NAME[];
extern const unsigned char CUT_ABBREV[];

extern const unsigned char COPY_NAME[];
extern const unsigned char COPY_ABBREV[];

extern const unsigned char PASTE_NAME[];
extern const unsigned char PASTE_ABBREV[];

extern const unsigned char ERASE_NAME[];
extern const unsigned char ERASE_ABBREV[];

extern const unsigned char MARKVERT_NAME[];
extern const unsigned char MARKVERT_ABBREV[];

extern const unsigned char PASTEVERT_NAME[];
extern const unsigned char PASTEVERT_ABBREV[];

extern const unsigned char OPENCLIP_NAME[];
extern const unsigned char OPENCLIP_ABBREV[];

extern const unsigned char SAVECLIP_NAME[];
extern const unsigned char SAVECLIP_ABBREV[];


extern const unsigned char FIND_NAME[];
extern const unsigned char FIND_ABBREV[];

extern const unsigned char FINDREGEXP_NAME[];
extern const unsigned char FINDREGEXP_ABBREV[];

extern const unsigned char REPLACE_NAME[];
extern const unsigned char REPLACE_ABBREV[];

extern const unsigned char REPLACEONCE_NAME[];
extern const unsigned char REPLACEONCE_ABBREV[];

extern const unsigned char REPLACEALL_NAME[];
extern const unsigned char REPLACEALL_ABBREV[];

extern const unsigned char REPEATLAST_NAME[];
extern const unsigned char REPEATLAST_ABBREV[];

extern const unsigned char GOTOLINE_NAME[];
extern const unsigned char GOTOLINE_ABBREV[];

extern const unsigned char GOTOCOLUMN_NAME[];
extern const unsigned char GOTOCOLUMN_ABBREV[];

extern const unsigned char GOTOMARK_NAME[];
extern const unsigned char GOTOMARK_ABBREV[];

extern const unsigned char MATCHBRACKET_NAME[];
extern const unsigned char MATCHBRACKET_ABBREV[];


extern const unsigned char RECORD_NAME[];
extern const unsigned char RECORD_ABBREV[];

extern const unsigned char PLAY_NAME[];
extern const unsigned char PLAY_ABBREV[];
extern const unsigned char PLAYONCE_ABBREV[];

extern const unsigned char OPENMACRO_NAME[];
extern const unsigned char OPENMACRO_ABBREV[];

extern const unsigned char SAVEMACRO_NAME[];
extern const unsigned char SAVEMACRO_ABBREV[];

extern const unsigned char MACRO_NAME[];
extern const unsigned char MACRO_ABBREV[];

extern const unsigned char UNLOADMACROS_NAME[];
extern const unsigned char UNLOADMACROS_ABBREV[];


extern const unsigned char REFRESH_NAME[];
extern const unsigned char REFRESH_ABBREV[];

extern const unsigned char UNDELLINE_NAME[];
extern const unsigned char UNDELLINE_ABBREV[];

extern const unsigned char CENTER_NAME[];
extern const unsigned char CENTER_ABBREV[];

extern const unsigned char PARAGRAPH_NAME[];
extern const unsigned char PARAGRAPH_ABBREV[];

extern const unsigned char TOUPPER_NAME[];
extern const unsigned char TOUPPER_ABBREV[];

extern const unsigned char TOLOWER_NAME[];
extern const unsigned char TOLOWER_ABBREV[];

extern const unsigned char CAPITALIZE_NAME[];
extern const unsigned char CAPITALIZE_ABBREV[];


extern const unsigned char MOVELEFT_NAME[];
extern const unsigned char MOVELEFT_ABBREV[];

extern const unsigned char MOVERIGHT_NAME[];
extern const unsigned char MOVERIGHT_ABBREV[];

extern const unsigned char LINEUP_NAME[];
extern const unsigned char LINEUP_ABBREV[];

extern const unsigned char LINEDOWN_NAME[];
extern const unsigned char LINEDOWN_ABBREV[];

extern const unsigned char PREVPAGE_NAME[];
extern const unsigned char PREVPAGE_ABBREV[];

extern const unsigned char NEXTPAGE_NAME[];
extern const unsigned char NEXTPAGE_ABBREV[];

extern const unsigned char MOVEEOL_NAME[];
extern const unsigned char MOVEEOL_ABBREV[];

extern const unsigned char MOVESOL_NAME[];
extern const unsigned char MOVESOL_ABBREV[];

extern const unsigned char MOVEEOF_NAME[];
extern const unsigned char MOVEEOF_ABBREV[];

extern const unsigned char MOVESOF_NAME[];
extern const unsigned char MOVESOF_ABBREV[];

extern const unsigned char TOGGLESEOF_NAME[];
extern const unsigned char TOGGLESEOF_ABBREV[];

extern const unsigned char TOGGLESEOL_NAME[];
extern const unsigned char TOGGLESEOL_ABBREV[];

extern const unsigned char NEXTWORD_NAME[];
extern const unsigned char NEXTWORD_ABBREV[];

extern const unsigned char PREVWORD_NAME[];
extern const unsigned char PREVWORD_ABBREV[];

extern const unsigned char MOVEEOW_NAME[];
extern const unsigned char MOVEEOW_ABBREV[];

extern const unsigned char SETBOOKMARK_NAME[];
extern const unsigned char SETBOOKMARK_ABBREV[];

extern const unsigned char GOTOBOOKMARK_NAME[];
extern const unsigned char GOTOBOOKMARK_ABBREV[];


extern const unsigned char TABSIZE_NAME[];
extern const unsigned char TABSIZE_ABBREV[];

extern const unsigned char INSERT_NAME[];
extern const unsigned char INSERT_ABBREV[];

extern const unsigned char FREEFORM_NAME[];
extern const unsigned char FREEFORM_ABBREV[];

extern const unsigned char STATUSBAR_NAME[];
extern const unsigned char STATUSBAR_ABBREV[];

extern const unsigned char SEARCHBACK_NAME[];
extern const unsigned char SEARCHBACK_ABBREV[];

extern const unsigned char CASESEARCH_NAME[];
extern const unsigned char CASESEARCH_ABBREV[];

extern const unsigned char FASTGUI_NAME[];
extern const unsigned char FASTGUI_ABBREV[];

extern const unsigned char WORDWRAP_NAME[];
extern const unsigned char WORDWRAP_ABBREV[];

extern const unsigned char RIGHTMARGIN_NAME[];
extern const unsigned char RIGHTMARGIN_ABBREV[];

extern const unsigned char AUTOINDENT_NAME[];
extern const unsigned char AUTOINDENT_ABBREV[];

extern const unsigned char AUTOPREFS_NAME[];
extern const unsigned char AUTOPREFS_ABBREV[];

extern const unsigned char BINARY_NAME[];
extern const unsigned char BINARY_ABBREV[];

extern const unsigned char TURBO_NAME[];
extern const unsigned char TURBO_ABBREV[];

extern const unsigned char NOFILEREQ_NAME[];
extern const unsigned char NOFILEREQ_ABBREV[];

extern const unsigned char VERBOSEMACROS_NAME[];
extern const unsigned char VERBOSEMACROS_ABBREV[];

extern const unsigned char DOUNDO_NAME[];
extern const unsigned char DOUNDO_ABBREV[];

extern const unsigned char READONLY_NAME[];
extern const unsigned char READONLY_ABBREV[];

extern const unsigned char CLIPNUMBER_NAME[];
extern const unsigned char CLIPNUMBER_ABBREV[];

extern const unsigned char LOADPREFS_NAME[];
extern const unsigned char LOADPREFS_ABBREV[];

extern const unsigned char SAVEPREFS_NAME[];
extern const unsigned char SAVEPREFS_ABBREV[];

extern const unsigned char LOADAUTOPREFS_NAME[];
extern const unsigned char LOADAUTOPREFS_ABBREV[];

extern const unsigned char SAVEAUTOPREFS_NAME[];
extern const unsigned char SAVEAUTOPREFS_ABBREV[];

extern const unsigned char SAVEDEFPREFS_NAME[];
extern const unsigned char SAVEDEFPREFS_ABBREV[];


extern const unsigned char INSERTCHAR_NAME[];
extern const unsigned char INSERTCHAR_ABBREV[];

extern const unsigned char DELETECHAR_NAME[];
extern const unsigned char DELETECHAR_ABBREV[];

extern const unsigned char BACKSPACE_NAME[];
extern const unsigned char BACKSPACE_ABBREV[];

extern const unsigned char INSERTLINE_NAME[];
extern const unsigned char INSERTLINE_ABBREV[];

extern const unsigned char DELETELINE_NAME[];
extern const unsigned char DELETELINE_ABBREV[];

extern const unsigned char DELETEEOL_NAME[];
extern const unsigned char DELETEEOL_ABBREV[];


extern const unsigned char UNDO_NAME[];
extern const unsigned char UNDO_ABBREV[];

extern const unsigned char REDO_NAME[];
extern const unsigned char REDO_ABBREV[];


extern const unsigned char HELP_NAME[];
extern const unsigned char HELP_ABBREV[];

extern const unsigned char BEEP_NAME[];
extern const unsigned char BEEP_ABBREV[];

extern const unsigned char FLASH_NAME[];
extern const unsigned char FLASH_ABBREV[];

extern const unsigned char EXEC_NAME[];
extern const unsigned char EXEC_ABBREV[];

extern const unsigned char SYSTEM_NAME[];
extern const unsigned char SYSTEM_ABBREV[];

extern const unsigned char ESCAPE_NAME[];
extern const unsigned char ESCAPE_ABBREV[];


/* This is the NULL-terminated, ordered list of names, useful for help etc. */

const unsigned char * const command_names[ACTION_COUNT+1];
