/* Error message vector.

   Copyright (C) 1993 Sebastiano Vigna

    This file is part of ne, the nice editor.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

In other words, you are welcome to use, share and improve this program.
You are forbidden to forbid anyone else to use, share and improve
what you give them.   Help stamp out software-hoarding!  */


#include "errors.h"

/* Whenever this vector is updated, the corresponding enum in errors.h
*must* be updated too. */

unsigned char *error_msg[ERROR_COUNT] = {
	"",
	"Syntax error.",
	"Not found.",
	"Can't save a document. Exit suspended.",
	"You are not positioned over {}, (), [] or <>.",
	"Can't find matching bracket.",
	"Bookmark number out of range.",
	"You cannot insert a character whose ASCII code is 0.",
	"No search string.",
	"No replace string.",
	"TAB size out of range.",
	"Mark a block first.",
	"Out of memory. DANGER!",
	"Nothing to undo.",
	"Nothing to redo",
	"No such command.",
	"Can execute only preference commands.",
	"Command needs a numeric argument.",
	"Command has no arguments.",
	"Command requires an argument.",
	"Wrong character after backslash",
	"Can't open file. Check existence and permissions.",
	"Error while writing.",
	"Document name has no extension.",
	"Can'f find or create $HOME/.ne directory.",
	"Clip does not exist.",
	"Mark is out of document.",
	"There is no text in the selected block.",
	"Can't open macro.",
	"This file is read-only.",
	"Stopped.",
	"I/O error.",
	"The argument string is empty."
};

unsigned char *info_msg[INFO_COUNT] = {
	"Saving...",
	"Done."
};
